@@ -54,7 +54,7 @@ Any other non--nil value update after confirmation."
 
 (defcustom helm-source-multi-occur-actions
   '(("Go to Line" . helm-moccur-goto-line)
-    ("Goto line other window" . helm-moccur-goto-line-ow)
+    ("Goto line other window (C-u vertically)" . helm-moccur-goto-line-ow)
     ("Goto line new frame" . helm-moccur-goto-line-of))
   "Actions for helm-occur and helm-moccur."
   :group 'helm-regexp
@@ -66,11 +66,35 @@ Any other non--nil value update after confirmation."
   :type 'boolean)
 
 (defcustom helm-moccur-show-buffer-fontification nil
-  "Show fontification of searched buffer in results when non nil."
+  "Show fontification of searched buffer in results when non nil.
+
+This enable or disable fontification globally in results, but you can
+override this default setting with `helm-moccur-buffer-substring-fn-for-modes'."
   :group 'helm-regexp
   :type '(radio :tag "Allow preserving fontification of searched buffer in results"
                 (const :tag "Don't preserve buffer fontification" nil)
                 (const :tag "Preserve buffer fontification" t)))
+
+(defcustom helm-moccur-buffer-substring-fn-for-modes
+  '((mu4e-headers-mode . buffer-substring)
+    (package-menu-mode . buffer-substring-no-properties))
+  "Alist that allow configuring the function to use for storing a buffer.
+
+Can be one of `buffer-substring' or `buffer-substring-no-properties'.
+Allow overriding the global effect of `helm-moccur-show-buffer-fontification'
+for a specific mode."
+  :group 'helm-regexp
+  :type '(alist :key-type (symbol :tag "Mode")
+                :value-type (radio (const :tag "With text properties" buffer-substring)
+                                   (const :tag "Without text properties" buffer-substring-no-properties))))
+
+(defcustom helm-occur-show-buffer-name nil
+  "Show buffer name in `helm-occur' results when non-nil.
+
+Not that this doesn't affect `helm-moccur' results and
+`helm-moccur-mode' buffers where buffer names are always shown."
+  :group 'helm-regexp
+  :type 'boolean)
 
 (defface helm-moccur-buffer
     '((t (:foreground "DarkTurquoise" :underline t)))
@@ -192,6 +216,10 @@ i.e Don't replace inside a word, regexp is surrounded with \\bregexp\\b."
 ;; Internal
 (defvar helm-multi-occur-buffer-list nil)
 (defvar helm-multi-occur-buffer-tick nil)
+(defvar helm-occur--invisible nil
+  "[INTERNAL] Hide buffer name in results when non-nil.
+Should be a local var to helm-buffer to allow resuming.")
+
 (defun helm-moccur-init ()
   "Create the initial helm multi occur buffer."
   (helm-init-candidates-in-buffer
@@ -201,6 +229,9 @@ i.e Don't replace inside a word, regexp is surrounded with \\bregexp\\b."
                                    #'buffer-substring #'buffer-substring-no-properties)
              for buf in buffers
              for bufstr = (with-current-buffer buf
+                            (helm-aif (assq major-mode
+                                            helm-moccur-buffer-substring-fn-for-modes)
+                                (setq bsubstring (cdr it)))
                             ;; A leading space is needed to allow helm
                             ;; searching the first line of buffer
                             ;; (#1725).
@@ -256,7 +287,7 @@ arg METHOD can be one of buffer, buffer-other-window, buffer-other-frame."
          (split-pat (helm-mm-split-pattern helm-input)))
     (cl-case method
       (buffer              (switch-to-buffer buf))
-      (buffer-other-window (switch-to-buffer-other-window buf))
+      (buffer-other-window (helm-display-buffers-other-windows buf))
       (buffer-other-frame  (switch-to-buffer-other-frame buf)))
     (with-current-buffer buf
       (helm-goto-line lineno)
@@ -395,7 +426,7 @@ Same as `helm-moccur-goto-line' but go in new frame."
                 (setq helm-multi-occur-buffer-tick new-tick-ls)
                 (message "Helm (m)occur Buffer have been udated")))))))))
 
-(defun helm-moccur-filter-one-by-one (candidate)
+(defun helm-moccur-filter-one-by-one (candidate &optional outside-helm)
   "`filter-one-by-one' function for `helm-source-moccur'."
   (require 'helm-grep)
   (let* ((split  (helm-grep-split-line candidate))
@@ -404,11 +435,16 @@ Same as `helm-moccur-goto-line' but go in new frame."
          (str    (nth 2 split)))
     (cons (concat (propertize
                    buf
+                   'invisible (and (null outside-helm)
+                                   (with-helm-buffer
+                                     helm-occur--invisible))
                    'face 'helm-moccur-buffer
                    'help-echo (buffer-file-name
                                (get-buffer buf))
                    'buffer-name buf)
-                  ":"
+                  (propertize ":" 'invisible (and (null outside-helm)
+                                                  (with-helm-buffer
+                                                    helm-occur--invisible)))
                   (propertize lineno 'face 'helm-grep-lineno)
                   ":"
                   (helm-grep-highlight-match str t))
@@ -525,7 +561,8 @@ Same as `helm-moccur-goto-line' but go in new frame."
                             (get-buffer (get-text-property
                                          (point) 'buffer-name)))
                            "\nmouse-1: set point\nmouse-2: jump to selection")
-               mouse-face highlight))
+               mouse-face highlight
+               invisible nil))
             (define-key map [mouse-1] 'mouse-set-point)
             (define-key map [mouse-2] 'helm-moccur-mode-mouse-goto-line)
             (define-key map [mouse-3] 'ignore)
@@ -597,7 +634,7 @@ Special commands:
                      do (with-current-buffer buffer
                           (insert
                             (propertize
-                             (car (helm-moccur-filter-one-by-one line))
+                             (car (helm-moccur-filter-one-by-one line t))
                              'helm-realvalue line)
                            "\n")))))
         (message "Reverting buffer done")))))
@@ -634,14 +671,16 @@ Special commands:
      'helm-multi-occur-buffer-tick
      (cl-loop for b in bufs
               collect (buffer-chars-modified-tick (get-buffer b)))))
+  (helm-set-local-variable 'helm-occur--invisible
+                           (null helm-occur-show-buffer-name))
   (helm :sources 'helm-source-occur
-        :buffer "*helm occur*"
-        :default (helm-aif (thing-at-point 'symbol) (regexp-quote it))
-        :history 'helm-occur-history
-        :preselect (and (memq 'helm-source-occur helm-sources-using-default-as-input)
-                        (format "%s:%d:" (regexp-quote (buffer-name))
-                                (line-number-at-pos (point))))
-        :truncate-lines helm-moccur-truncate-lines))
+          :buffer "*helm occur*"
+          :default (helm-aif (thing-at-point 'symbol) (regexp-quote it))
+          :history 'helm-occur-history
+          :preselect (and (memq 'helm-source-occur helm-sources-using-default-as-input)
+                          (format "%s:%d:" (regexp-quote (buffer-name))
+                                  (line-number-at-pos (point))))
+          :truncate-lines helm-moccur-truncate-lines))
 
 ;;;###autoload
 (defun helm-occur-from-isearch ()
@@ -659,6 +698,8 @@ Special commands:
      'helm-multi-occur-buffer-tick
      (cl-loop for b in bufs
               collect (buffer-chars-modified-tick (get-buffer b))))
+    (helm-set-local-variable 'helm-occur--invisible
+                             (null helm-occur-show-buffer-name))
     (helm :sources 'helm-source-occur
           :buffer "*helm occur*"
           :history 'helm-occur-history
