@@ -142,6 +142,9 @@ Same as `helm-kill-selection-and-quit' called with a prefix arg."
 
 (defun helm-kill-ring-action-yank (str)
   "Insert STR in `kill-ring' and set STR to the head.
+
+When called with a prefix arg, point and mark are exchanged without
+activating region.
 If this action is executed just after `yank',
 replace with STR as yanked string."
   (with-helm-current-buffer
@@ -155,7 +158,22 @@ replace with STR as yanked string."
            (when (and (region-active-p) delete-selection-mode)
              (delete-region (region-beginning) (region-end)))
            (if (not (eq (helm-attr 'last-command helm-source-kill-ring) 'yank))
-               (insert-for-yank str)
+               (progn
+                 ;; Ensure mark is at beginning of inserted text.
+                 (push-mark)
+                 ;; When yanking in a helm minibuffer we need a small
+                 ;; delay to detect the mark in previous minibuffer. [1]
+                 (run-with-timer
+                  0.01 nil
+                  (lambda ()
+                    (insert-for-yank str)
+                    (when helm-current-prefix-arg
+                      ;; Same as exchange-point-and-mark but without
+                      ;; activating region.
+                      (goto-char (prog1 (mark t)
+                                   (set-marker (mark-marker)
+                                               (point)
+                                               helm-current-buffer)))))))
                ;; from `yank-pop'
                (let ((inhibit-read-only t)
                      (before (< (point) (mark t))))
@@ -164,7 +182,8 @@ replace with STR as yanked string."
                      (funcall (or yank-undo-function 'delete-region) (mark t) (point)))
                  (setq yank-undo-function nil)
                  (set-marker (mark-marker) (point) helm-current-buffer)
-                 (insert-for-yank str)
+                 ;; Same as [1]
+                 (run-with-timer 0.01 nil (lambda () (insert-for-yank str)))
                  ;; Set the window start back where it was in the yank command,
                  ;; if possible.
                  (set-window-start (selected-window) yank-window-start t)
@@ -283,43 +302,6 @@ This is a command for `helm-kill-ring-map'."
                when (and gm (not (assoc gm recip)))
                collect (cons gm marker) into recip
                finally return recip))))
-
-(defun helm--push-mark (&optional location nomsg activate)
-  "[Internal] Don't use directly, use instead `helm-push-mark-mode'."
-  (unless (null (mark t))
-    (setq mark-ring (cons (copy-marker (mark-marker)) mark-ring))
-    (when (> (length mark-ring) mark-ring-max)
-      (move-marker (car (nthcdr mark-ring-max mark-ring)) nil)
-      (setcdr (nthcdr (1- mark-ring-max) mark-ring) nil)))
-  (set-marker (mark-marker) (or location (point)) (current-buffer))
-  ;; Now push the mark on the global mark ring.
-  (setq global-mark-ring (cons (copy-marker (mark-marker))
-                               ;; Avoid having multiple entries
-                               ;; for same buffer in `global-mark-ring'.
-                               (cl-loop with mb = (current-buffer)
-                                        for m in global-mark-ring
-                                        for nmb = (marker-buffer m)
-                                        unless (eq mb nmb)
-                                        collect m)))
-  (when (> (length global-mark-ring) global-mark-ring-max)
-    (move-marker (car (nthcdr global-mark-ring-max global-mark-ring)) nil)
-    (setcdr (nthcdr (1- global-mark-ring-max) global-mark-ring) nil))
-  (or nomsg executing-kbd-macro (> (minibuffer-depth) 0)
-      (message "Mark set"))
-  (when (or activate (not transient-mark-mode))
-    (set-mark (mark t)))
-  nil)
-
-;;;###autoload
-(define-minor-mode helm-push-mark-mode
-    "Provide an improved version of `push-mark'.
-Modify the behavior of `push-mark' to update
-the `global-mark-ring' after each new visit."
-  :group 'helm-ring
-  :global t
-  (if helm-push-mark-mode
-      (advice-add 'push-mark :override #'helm--push-mark)
-      (advice-remove 'push-mark #'helm--push-mark)))
 
 ;;;; <Register>
 ;;; Insert from register
