@@ -113,7 +113,7 @@ This affect also sorting functions in the same way."
     (cancel-timer helm-top--poll-timer))
   (condition-case nil
       (progn
-        (when (and (helm-alive-p) (not (helm-action-window)) (null no-update))
+        (when (and (helm--alive-p) (null no-update))
           ;; Fix quitting while process is running
           ;; by binding `with-local-quit' in init function
           ;; Issue #1521.
