@@ -20,8 +20,7 @@
 (require 'cl-lib)
 (require 'helm)
 (require 'helm-help)
-(require 'compile) ; Fixme: Is this needed?
-(require 'dired)
+(eval-when-compile (require 'dired))
 
 (declare-function helm-find-files-1 "helm-files.el" (fname &optional preselect))
 (declare-function popup-tip "ext:popup")
@@ -64,7 +63,7 @@ It is a float, usually 1024.0 but could be 1000.0 on some systems."
   :type 'integer)
 
 (defcustom helm-sources-using-help-echo-popup '("Moccur" "Imenu in all buffers"
-                                                "Ack-Grep" "AG" "Gid" "Git-Grep")
+                                                "Ack-Grep" "AG" "RG" "Gid" "Git-Grep")
   "Show the buffer name or the filename in a popup at selection."
   :group 'helm-utils
   :type '(repeat (choice string)))
@@ -218,30 +217,104 @@ In this case last position is added to the register
 ;;; Utils functions
 ;;
 ;;
+(defcustom helm-switch-to-buffer-ow-vertically nil
+  "Maybe switch to other window vertically when non nil.
+
+Possible values are `t', `nil' and `decide'.
+
+When `t' switch vertically.
+When nil switch horizontally.
+When `decide' try to guess if it is possible to switch vertically
+according to the setting of `split-width-threshold' and the size of
+the window from where splitting is done.
+
+Note that when using `decide' and `split-width-threshold' is nil, the
+behavior is the same that with a nil value."
+  :group 'helm-utils
+  :type '(choice
+           (const :tag "Split window vertically" t)
+           (const :tag "Split window horizontally" nil)
+           (symbol :tag "Guess how to split window" 'decide)))
+
 (defun helm-switch-to-buffers (buffer-or-name &optional other-window)
   "Switch to buffer BUFFER-OR-NAME.
+
 If more than one buffer marked switch to these buffers in separate windows.
 If OTHER-WINDOW is specified keep current-buffer and switch to others buffers
-in separate windows."
-  (let* ((mkds (helm-marked-candidates))
-         (size (/ (window-height) (length mkds))))
-    (or (<= window-min-height size)
-        (error "Too many buffers to visit simultaneously."))
+in separate windows.
+If a prefix arg is given split windows vertically."
+  (let ((mkds          (helm-marked-candidates))
+        (initial-ow-fn (if (cdr (window-list))
+                           #'switch-to-buffer-other-window
+                         #'helm-switch-to-buffer-other-window)))
     (helm-aif (cdr mkds)
         (progn
           (if other-window
-              (switch-to-buffer-other-window (car mkds))
+              (funcall initial-ow-fn (car mkds))
             (switch-to-buffer (car mkds)))
           (save-selected-window
-            (cl-loop for b in it
-                  do (progn
-                       (select-window (split-window))
-                       (switch-to-buffer b)))))
+            (cl-loop with nosplit
+                     for b in it
+                     when nosplit return
+                     (message "Too many buffers to visit simultaneously")
+                     do (condition-case _err
+                            (helm-switch-to-buffer-other-window b 'balance)
+                          (error (setq nosplit t) nil)))))
       (if other-window
-          (switch-to-buffer-other-window buffer-or-name)
+          (funcall initial-ow-fn buffer-or-name)
         (switch-to-buffer buffer-or-name)))))
 
-(defun helm-switch-to-buffers-other-window (buffer-or-name)
+(defun helm-simultaneous-find-file (files &optional other-window)
+  "Find files in FILES list in separate windows.
+If frame is too small to display all windows, continue finding files
+in background.
+When called with a prefix arg split is done vertically."
+  (helm-aif (cdr files)
+      (progn
+        (unless other-window
+          (switch-to-buffer (find-file-noselect (car files))))
+        (save-selected-window
+          (cl-loop with nosplit
+                   with len = (length it)
+                   with remaining = 0
+                   with displayed = 0
+                   for f in it
+                   for count from 1
+                   for buf = (find-file-noselect f)
+                   unless nosplit do
+                   (condition-case-unless-debug _err
+                       (helm-switch-to-buffer-other-window buf 'balance)
+                     (error
+                      (setq nosplit t)
+                      (message
+                       "%d files displayed, %d files opening in background..."
+                       (setq displayed count)
+                       (setq remaining (- len count)))
+                      nil))
+                   finally
+                   (when nosplit
+                     (message
+                      "%d files displayed, %d files opened in background"
+                      displayed remaining)))))))
+
+(defun helm-switch-to-buffer-other-window (buffer-or-name &optional balance)
+  "Switch to buffer-or-name in other window.
+If a prefix arg is detected split vertically.
+When argument balance is provided `balance-windows'."
+  (let* ((helm-switch-to-buffer-ow-vertically
+          (if (eq helm-switch-to-buffer-ow-vertically 'decide)
+              (and (numberp split-width-threshold)
+                   (>= (window-width (selected-window))
+                       split-width-threshold))
+            helm-switch-to-buffer-ow-vertically))
+         (right-side (if helm-switch-to-buffer-ow-vertically
+                        (not helm-current-prefix-arg)
+                      helm-current-prefix-arg)))
+    (select-window (split-window nil nil right-side))
+    (and balance (balance-windows))
+    (switch-to-buffer buffer-or-name)))
+
+(defun helm-display-buffers-other-windows (buffer-or-name)
   "switch to buffer BUFFER-OR-NAME in other window.
 See `helm-switch-to-buffers' for switching to marked buffers."
   (helm-switch-to-buffers buffer-or-name t))
@@ -431,15 +504,6 @@ that is sorting is done against real value of candidate."
            (< (length str1) (length str2)))
           (t (> sc1 sc2)))))
 
-(defun helm-ff-get-host-from-tramp-invalid-fname (fname)
-  "Extract hostname from an incomplete tramp file name.
-Return nil on valid file name remote or not."
-  (let* ((str (helm-basename fname))
-         (split (split-string str ":" t))
-         (meth (car (member (car split)
-                            (helm-ff-get-tramp-methods))))) 
-    (when meth (car (last split)))))
-
 (cl-defun helm-file-human-size (size &optional (kbsize helm-default-kbsize))
   "Return a string showing SIZE of a file in human readable form.
 SIZE can be an integer or a float depending it's value.
@@ -561,6 +625,13 @@ If STRING is non--nil return instead a space separated string."
             (mapconcat 'identity (list type user group other) " ")
           (list :mode-type type :user user :group group :other other))))
 
+(defun helm-format-columns-of-files (files)
+  "Same as `dired-format-columns-of-files'.
+Inlined here for compatibility."
+  (let ((beg (point)))
+    (completion--insert-strings files)
+    (put-text-property beg (point) 'mouse-face nil)))
+
 (defmacro with-helm-display-marked-candidates (buffer-or-name candidates &rest body)
   (declare (indent 0) (debug t))
   (helm-with-gensyms (buffer window)
@@ -573,7 +644,7 @@ If STRING is non--nil return instead a space separated string."
             helm-reuse-last-window-split-state
             ,window)
        (with-current-buffer ,buffer
-         (dired-format-columns-of-files ,candidates))
+         (helm-format-columns-of-files ,candidates))
        (unwind-protect
             (with-selected-window
                 (setq ,window (temp-buffer-window-show
